<?php

namespace MJ\LoginAsUser\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class LoginAsUser extends AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$session = $this->session();
		if(!$session->get('mjlauUserId')){
			/** @var \XF\Entity\User $visitor */
			$visitor = \XF::visitor();

			if (!$visitor->hasPermission('general', 'mjlauLoginAsUser'))
			{
				throw $this->exception($this->noPermission());
			}
		}
	}

	public function actionIndex()
	{
		$input = $this->filter([
			'user_id'    => 'uint',
			'username'    => 'str',
			'reason'      => 'str',
		]);

		if ($this->isPost())
		{
			$visitor = \XF::visitor();
			$userNotFound = true;
			if($this->options()->xfrmResourcesPerPage && !$input['reason']){
				return $this->error(\XF::phrase('mjlau_please_enter_reason_to_login'));
			}
			if ($input['user_id'])
			{
				$user = $this->em()->find('XF:User', $input['user_id']);
			}
			else
			{
				$user = $this->em()->findOne('XF:User', ['username' => $input['username']]);
			}

			if($user && $user->user_id == $visitor->user_id){
				return $this->error(\XF::phrase('mjlau_you_already_logged_in'));
			}
			if ($user)
			{
				$userNotFound = false;
			}
			if($userNotFound){
				return $this->error(\XF::phrase('requested_user_not_found'));
			}

			if($this->options()->mjlauDisallowedUserIds){
				$disallowedIds = preg_split('/\s+/', trim($this->options()->mjlauDisallowedUserIds), -1, PREG_SPLIT_NO_EMPTY);
				if($disallowedIds && in_array($user->user_id, $disallowedIds)){
					return $this->error(\XF::phrase('mjlau_you_cannot_login_as_x', ['username' => $user->username]));
				}
			}

			$session = $this->session();
			if($session->get('mjlauUserId')){
				$mjlauUserId = $session->get('mjlauUserId');
				$mjlauUsername = $session->get('mjlauUsername');
				if($user->user_id == $mjlauUserId){
					return $this->actionLogout();
				}
			}else{
				$mjlauUserId = $visitor->user_id;
				$mjlauUsername = $visitor->username;
			}

			/** @var \XF\ControllerPlugin\Login $loginPlugin */
			$loginPlugin = $this->plugin('XF:Login');
			$loginPlugin->completeLogin($user, 0);

			$session = $this->session();
			$session->mjlauUserId = $mjlauUserId;
			$session->mjlauUsername = $mjlauUsername;
			$loginData = [
				'user_id'       => $mjlauUserId,
				'login_user_id' => $user->user_id,
				'reason'        => $input['reason'],
				'ip_address'    => \XF\Util\Ip::convertIpStringToBinary($this->app()->request()->getIp()),
			];

			$loginLog = $this->em()->create('MJ\LoginAsUser:LoginLog');
			$loginLog->bulkSet($loginData);
			$loginLog->save();

			$redirect = $this->getDynamicRedirectIfNot($this->buildLink('login'));

			return $this->redirect($redirect, '');
		}else{
			$viewParams = [
				'username'             => $input['username'],
				'reason'               => $input['reason'],
			];

			return $this->view('MJ\LoginAsUser:SendCredit', 'mjlau_login_as_user', $viewParams);
		}
	}

	public function actionLogout()
	{
		$session = $this->session();

		if ($session->exists())
		{
			$userId = $session->get('mjlauUserId');
			$user = $this->em()->find('XF:User', $userId);

			/** @var \XF\ControllerPlugin\Login $loginPlugin */
			$loginPlugin = $this->plugin('XF:Login');
			$loginPlugin->completeLogin($user, 0);

			$session = $this->session();

			unset($session->mjlauUserId);
			unset($session->mjlauUsername);
		}
		$redirect = $this->getDynamicRedirectIfNot($this->buildLink('login'));

		return $this->redirect($redirect, '');
	}
}
